/*********************************************************** {COPYRIGHT-TOP-OCO} ***
// IBM Confidential
//
// OCO Source Materials
//
// 5724-C51
//
// (C) Copyright IBM Corp. 2004 All Rights Reserved.
//
// The source code for this program is not published or other-
// wise divested of its trade secrets, irrespective of what has
// been deposited with U.S. Copyright Office.
************************************************************ {COPYRIGHT-END-OCO} **/
/* Change Activity on Version 1.7 Dated 7/19/04:
* @00=ES,V3R1, 01/20/2004, sjl: Initial version
*******************************************************************************/
package com.ibm.tbsm.server.comp.es.sample;

import java.util.*;
import java.io.*;
import java.util.Date.*;

import com.ibm.tbsm.server.cmn.external.*;
import com.ibm.tbsm.server.comp.es.external.*;
import com.ibm.tbsm.server.comp.es.*;
import com.ibm.tbsm.server.svc.log.external.*;
import com.ibm.tbsm.server.es.nls.*;

/*******************************************************************************
* This class establishes a connection to Event Enablement and receives, processes
* and stores event data.  It will run continuously until signaled to
* terminate.
*******************************************************************************/
public class EIFSenderUserExit extends Thread implements IEventSenderUserExit
{
    private static final String  COPYRIGHT  = com.ibm.tbsm.server.cmn.external.CmnCopyRight.TBSM_COPYRIGHT;
    private static final String  CLASS_NAME = "EIFSenderUserExit";
    private static final boolean DEBUG      = CmnConstants.DEVELOPMENT_TIME &&
                                              CmnUnitTest.getDebugMode(CLASS_NAME);

    /**************************************************************************
    * Constructs a new EIFSenderUserExit.
    *
    * @param es reference to EsJdbcHandlerDb
    **************************************************************************/
    public EIFSenderUserExit(EsJdbcHandlerDb jdbcHandler)
    {
        jdbcHandlerDb_     = jdbcHandler;

        jdbcHandlerDb_.logMessage("UserExit - Constructor...");

    }

    /**************************************************************************
    * This method establishes a connection to an Event Enabler and processes
    * events continuously until signaled to terminate.
    **************************************************************************/
    public void run()
    {

        jdbcHandlerDb_.logMessage("UserExit - Thread started...");

        int numberProcessed = 0;

        // Loop until terminateRequested
        while (!terminateRequested_)
        {

            ASITBSMEventState aStateEvent = (ASITBSMEventState) dequeueState();

            if (aStateEvent != null)
            {
                jdbcHandlerDb_.updateRequestStatus(aStateEvent.getGuid(),205);
                jdbcHandlerDb_.logMessage("UserExit - State Object dequeued: " + aStateEvent.getGuid());
                System.out.println("UserExit - State Object dequeued!!");

                numberProcessed++;
            }

            ASITBSMEventStatus aStatusEvent = (ASITBSMEventStatus) dequeueStatus();

            if (aStatusEvent != null)
            {
                jdbcHandlerDb_.updateRequestStatus(aStatusEvent.getGuid(),205);
                jdbcHandlerDb_.logMessage("UserExit - Status Object dequeued: " + aStatusEvent.getGuid());
                System.out.println("UserExit - Status Object dequeued!");
                numberProcessed++;
            }

            //Give other threads a chance
            if (((stateObjectQueue_.size() == 0) &&
                (statusObjectQueue_.size() == 0)) ||
                (numberProcessed > EE_MAX_EVENTS))
            {
                try
                {
                    numberProcessed = 0;
                    Thread.currentThread().sleep(EE_RUN_SLEEP);
                }
                catch (InterruptedException exs)
                {
                    // No action required
                }
            }

        }

        jdbcHandlerDb_.logMessage("UserExit - Thread stopped...");

        return;
    }


    /******************************************************************
    * Enqueue an object onto this stateObjectQueue.
    *
    * @param objectToAdd Object to enqueue.
    *****************************************************************/
    public synchronized void enqueueState( Object objectToAdd )
    {
        stateObjectQueue_.add(objectToAdd);
    }

    /******************************************************************
    * Enqueue an object onto this statusObjectQueue.
    *
    * @param objectToAdd Object to enqueue.
    *****************************************************************/
    public synchronized void enqueueStatus( Object objectToAdd )
    {
        statusObjectQueue_.add(objectToAdd);
    }

    /******************************************************************
    * Set Terminate flag to stop processing.
    *****************************************************************/
    public void setTerminateRequested( )
    {
        terminateRequested_ = true;
    }

    /******************************************************************
    * Dequeue an object off of this Queue. 
    *
    * @return First element currently on the queue.
    *****************************************************************/
    private Object dequeueState()
    {

        Object obj = null;

        if (stateObjectQueue_.size() > 0)
        {
            obj = stateObjectQueue_.remove(0);
        }
        return(obj);
    }

    /******************************************************************
    * Dequeue an object off of this Queue. 
    *
    * @return First element currently on the queue.
    *****************************************************************/
    private Object dequeueStatus()
    {

        Object obj = null;


        if (statusObjectQueue_.size() > 0)
        {
            obj = statusObjectQueue_.remove(0);
        }

        return(obj);
    }

    /**************************************************************************
    * Variables
    **************************************************************************/
    private static final int    EE_RUN_SLEEP                   = 10;
    private static final int    EE_MAX_EVENTS                  = 50;
    private   List              stateObjectQueue_              = new ArrayList();
    private   List              statusObjectQueue_             = new ArrayList();
    public    boolean           terminateRequested_            = false;
    protected EsJdbcHandlerDb   jdbcHandlerDb_                 = null;
}
